Class: Rdkafka::Admin::CreateAclHandle
- Inherits:
-
Rdkafka::AbstractHandle
- Object
- FFI::Struct
- Rdkafka::AbstractHandle
- Rdkafka::Admin::CreateAclHandle
- Defined in:
- lib/rdkafka/admin/create_acl_handle.rb
Overview
Handle for create ACL operation
Constant Summary
Constants inherited from Rdkafka::AbstractHandle
Rdkafka::AbstractHandle::REGISTRY
Instance Attribute Summary
Attributes inherited from Rdkafka::AbstractHandle
Instance Method Summary collapse
-
#create_result ⇒ CreateAclReport
Report prepared by the background event callback, with rdkafka_response value as 0 and rdkafka_response_string value as empty string if the acl creation was successful.
-
#operation_name ⇒ String
The name of the operation.
-
#raise_error ⇒ Object
Raises an error if the operation failed.
Methods inherited from Rdkafka::AbstractHandle
#initialize, #pending?, #prepared_result, register, remove, #unlock, #wait
Methods included from Helpers::Time
#monotonic_now, #monotonic_now_ms
Constructor Details
This class inherits a constructor from Rdkafka::AbstractHandle
Instance Method Details
#create_result ⇒ CreateAclReport
Returns report prepared by the background event callback, with rdkafka_response value as 0 and rdkafka_response_string value as empty string if the acl creation was successful.
18 19 20 |
# File 'lib/rdkafka/admin/create_acl_handle.rb', line 18 def create_result prepared_result end |
#operation_name ⇒ String
Returns the name of the operation.
11 12 13 |
# File 'lib/rdkafka/admin/create_acl_handle.rb', line 11 def operation_name "create acl" end |
#raise_error ⇒ Object
Raises an error if the operation failed
24 25 26 27 28 29 |
# File 'lib/rdkafka/admin/create_acl_handle.rb', line 24 def raise_error raise RdkafkaError.new( self[:response], broker_message: ) end |