Class: Rdkafka::Admin::DeleteAclHandle
- Inherits:
-
Rdkafka::AbstractHandle
- Object
- FFI::Struct
- Rdkafka::AbstractHandle
- Rdkafka::Admin::DeleteAclHandle
- Defined in:
- lib/rdkafka/admin/delete_acl_handle.rb
Overview
Handle for delete ACL operation
Constant Summary
Constants inherited from Rdkafka::AbstractHandle
Rdkafka::AbstractHandle::REGISTRY
Instance Attribute Summary
Attributes inherited from Rdkafka::AbstractHandle
Instance Method Summary collapse
-
#create_result ⇒ DeleteAclReport
Report prepared by the background event callback, with an array of matching_acls.
-
#operation_name ⇒ String
The name of the operation.
-
#raise_error ⇒ Object
Raises an error if the operation failed.
Methods inherited from Rdkafka::AbstractHandle
#initialize, #pending?, #prepared_result, register, remove, #unlock, #wait
Methods included from Helpers::Time
#monotonic_now, #monotonic_now_ms
Constructor Details
This class inherits a constructor from Rdkafka::AbstractHandle
Instance Method Details
#create_result ⇒ DeleteAclReport
Returns report prepared by the background event callback, with an array of matching_acls.
17 18 19 |
# File 'lib/rdkafka/admin/delete_acl_handle.rb', line 17 def create_result prepared_result end |
#operation_name ⇒ String
Returns the name of the operation.
11 12 13 |
# File 'lib/rdkafka/admin/delete_acl_handle.rb', line 11 def operation_name "delete acl" end |
#raise_error ⇒ Object
Raises an error if the operation failed
23 24 25 26 27 28 |
# File 'lib/rdkafka/admin/delete_acl_handle.rb', line 23 def raise_error raise RdkafkaError.new( self[:response], broker_message: ) end |