Class: Rdkafka::Admin::DescribeAclHandle
- Inherits:
-
Rdkafka::AbstractHandle
- Object
- FFI::Struct
- Rdkafka::AbstractHandle
- Rdkafka::Admin::DescribeAclHandle
- Defined in:
- lib/rdkafka/admin/describe_acl_handle.rb
Overview
Handle for describe ACL operation
Constant Summary
Constants inherited from Rdkafka::AbstractHandle
Rdkafka::AbstractHandle::REGISTRY
Instance Method Summary collapse
-
#create_result ⇒ DescribeAclReport
Instance with an array of acls that matches the request filters.
-
#operation_name ⇒ String
The name of the operation.
-
#raise_error ⇒ Object
Raises an error if the operation failed.
Methods inherited from Rdkafka::AbstractHandle
#initialize, #pending?, register, remove, #unlock, #wait
Methods included from Helpers::Time
#monotonic_now, #monotonic_now_ms
Constructor Details
This class inherits a constructor from Rdkafka::AbstractHandle
Instance Method Details
#create_result ⇒ DescribeAclReport
Returns instance with an array of acls that matches the request filters.
19 20 21 |
# File 'lib/rdkafka/admin/describe_acl_handle.rb', line 19 def create_result DescribeAclReport.new(acls: self[:acls], acls_count: self[:acls_count]) end |
#operation_name ⇒ String
Returns the name of the operation.
14 15 16 |
# File 'lib/rdkafka/admin/describe_acl_handle.rb', line 14 def operation_name "describe acl" end |
#raise_error ⇒ Object
Raises an error if the operation failed
25 26 27 28 29 30 |
# File 'lib/rdkafka/admin/describe_acl_handle.rb', line 25 def raise_error raise RdkafkaError.new( self[:response], broker_message: self[:response_string].read_string ) end |