Class: Rdkafka::Bindings::ConfigResource

Inherits:
FFI::Struct
  • Object
show all
Defined in:
lib/rdkafka/bindings.rb

Overview

FFI struct representing a config resource (rd_kafka_ConfigResource_t) Structs for management of configurations. Each configuration is attached to a resource and one resource can have many configuration details. Each resource will also have separate errors results if obtaining configuration was not possible for any reason